# *** PACK_GRID_HANDLERS PY - 8 Jul 2023  17:46:08 - JGKNAUTH

"""
pack_grid_handlers.py contains the handlers for two of the largest parts
of Pack Grid rocessing: the library listboxes and the information text
boxes.
"""

version = 2.2

from tkinter import *
from tkinter.messagebox import showerror, showwarning
import sys, os, subprocess
import pack_grid_utilities, pack_grid_documentation


# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$  Library Listboxes  $$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

# Create (or destroy and recreate) a window with a listbox to hold the
# names of the files in a test case library.  The recreation is needed
# to get the latest set of files in the directory in case that has
# changed since the last library display.  Also, if the user is
# switching between libraries, it is necessary to destroy the old
# listbox display so the new one can be built.

lib_dir = ''                  # Directory of the test case files library
lib_win = ''                  # Window for the library list box
listbx = ''                   # List box for the library files
display_function = None       # Display function in pack_grid.pyw

def open_library_window(
       lib_dir_in, lib_title, test_du_jour=None,
       dialog=None, display_func=None):

   global lib_dir, lib_win, listbx, display_function

   lib_dir = lib_dir_in             # Save as globals for other handlers
   display_function = display_func  #

   # Calculate settings for the window holding the Library listbox
   # object using geometry of the dialog window.
   dialog_width  = dialog.winfo_width()
   dialog_height = dialog.winfo_height()
   dialog_x_offset = dialog.winfo_x()
   dialog_y_offset = dialog.winfo_y()

   lib_width = 470
   lib_height = dialog_height
   lib_x_offset = dialog_x_offset + dialog_width + 20
   lib_y_offset = dialog_y_offset

   # Delete any old library window if it exists.
   if lib_win != '': lib_win.destroy()

   # Set up the new Toplevel window to house the new listbox display.
   lib_win = Toplevel()
   lib_win.title(lib_title)
   lib_win.focus_force()
   lib_win.protocol('WM_DELETE_WINDOW',lib_win_destroy_win)
   str_width = str(lib_width)
   str_height = str(lib_height)
   str_x_offset = str(lib_x_offset)
   str_y_offset = str(lib_y_offset)
   geom = str_width + 'x' + str_height + '+' + str_x_offset + '+' + str_y_offset
   lib_win.geometry(geom)

   # Build the header label.
   lbl = Label(lib_win,
            text='To execute a test case file from this list, you must *DOUBLE-CLICK* it\n' +
                 'with the *LEFT* mouse button.  To then close the displayed test case\n' +
                 'results, just *LEFT-DOUBLE-CLICK* anywhere in the results display.  That\n' +
                 'is the same as clicking the X in the upper right corner of the display.\n\n' +
                 'To instead text edit a file from this list, first SELECT it with a *single*\n' +
                 'LEFT-CLICK.  Then do a *single* RIGHT-CLICK.  Only .txt files can be\n' +
                 'edited this way; a warning will be issued for any other type of file.  The\n' +
                 'editor can be specified (with its full path) in a PACK_GRID_EDITOR\n' +
                 'environment variable. If no editor is specified, Windows file association\n' +
                 'will be used.',
               bg='cyan', font=('Lucida', 10, 'bold'), justify=LEFT)
   lbl.pack(side=TOP, fill=X)

   # Build the list box.
   sbar = Scrollbar(lib_win)
   listbx = Listbox(lib_win)
   sbar.config(command=listbx.yview)
   listbx.config(yscrollcommand=sbar.set)
   listbx.config(font=('Lucida', 9), bd=5, relief=FLAT, selectmode=SINGLE)
   sbar.pack(side=RIGHT, fill=Y)
   listbx.pack(side=LEFT, expand=YES, fill=BOTH)

   # Load the list box.
   pos = 0                        # Position for next listbox entry.
   du_jour_pos = 0                # Position where test_du_jour is found.
   found_du_jour = False
   library = os.listdir(lib_dir)

   if len(library) == 0: return
   for file in library:
      fqn = os.path.join(lib_dir,file)
      if os.path.isfile(fqn):
         listbx.insert(pos, file)
         if file == test_du_jour:
            du_jour_pos = pos
            found_du_jour = True
         pos += 1

   #Set listbox selection.
   if test_du_jour:
      listbx.selection_set(du_jour_pos)
   else:
      listbx.selection_set(0)

   # Make sure the du_jour test case exists if it was requested.
   if test_du_jour and not found_du_jour:
       listbx.selection_clear(0,END)
       showerror('ERROR!',
                 test_du_jour + ' could not be found in ' + lib_dir + '.')
       return

   # Set up execution and edit keys.
   listbx.bind('<Double-1>', handleList)
   listbx.bind('<Button-3>', lib_win_edit)
   return


# This is called when the user double-clicks an entry in the Library
# listbox with the intent of displaying the results for the selected
# test case file.
def handleList(event):
    global listbx, lib_dir, display_function
    index = listbx.curselection()
    file = listbx.get(index)
    fileid = os.path.join(lib_dir,file)   # User-selected file from the Library

    # $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    # Call the display_test_cases function in pack_grid.pyw to process
    # the selected file and display the test case results (via
    # pack_grid_utilities).  The reference to display_test_cases was
    # passed in the call to open_library_window and was then saved as
    # the display_function global variable to make it available here.
    #$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

    display_function(fileid)

    return

#-----------------------------------------------------------------------
# This is called when the user right-clicks an entry in the Library
# listbox with the intent of editing the selected file.
def lib_win_edit(event):
    global listbx, lib_dir
    index = listbx.curselection()
    if index == ():
       showwarning('WARNING!',
                   'Be sure to first *select* some file with a LEFT click before trying to ' +
                   'edit that file with a RIGHT click.')
       return

    file = listbx.get(index)
    (name, ext) = os.path.splitext(file)
    if ext.upper() != ".TXT":
       showwarning('WARNING!',
                   'Directory  ' + lib_dir + '\n\n' +
                   'Filename:  ' + file + '\n\n' +
                   'This is not a .txt file.  As a precaution, the simulated left ' +
                   'double-click (file extension association execution) will be ignored.')
       return

    # The surrounding quotes are needed in Windows.  Probably also in UNIX.
    good_file = '"' + os.path.join(lib_dir,file) + '"' # User-selected library file
    editor = os.getenv('PACK_GRID_EDITOR','')          # Preferred editor (or none)

    try:
       if editor == '' or editor == '""' or editor == "''":
          # If no specified editor, just go edit the file in Windows using
          # file associations.
          os.startfile(good_file)      # This is a Windows-only function
       else:
          # This *should* work in UNIX as well as in Windows.
          subprocess.Popen(editor + " " + good_file)

    except OSError as X:
       showerror('ERROR',
                   'Directory  ' + lib_dir + '\n\n' +
                   'Filename:  ' + file + '\n\n' +
                   str(X))
    return


# ----------------------------------------------------------------------
# This is called when the user closes the window of the Library listbox.
def lib_win_destroy_win():
    global lib_win
    lib_win.destroy()
    return


# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$$$$$$$$$$$$$$$$  Information Text Boxes  $$$$$$$$$$$$$$$$$$$$$$$
# $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

# This set of functions processes the user selection from the info
# dialog that was set up by pack_grid.pyw.  A Toplevel window is built
# and the selected documentation is then formatted and displayed in it.

general_info_displayed = False
data_structures_displayed = False
top1 = ''; top2 = ''

info =  ['Display general information about the Pack Grid tool.',
         'Display information about the main data structures.']

ginfo_var = StringVar()

# This is called when one of the program information radio buttons was
# pushed.
def ginfo_onPress():
    global general_info_displayed, data_structures_displayed, top1, top2

    pick = ginfo_var.get()

    if pick == info[0]:                   # Display general info on tool
       if general_info_displayed == True:
          top1.focus_force()
          return
       general_info_displayed = True

       top1 = Toplevel()
       top1.title('General Information about Pack Grid')
       top1.focus_force()
       display_info(top1, 'General_Information')

    elif pick == info[1]:              # Display info on data structures
       if data_structures_displayed == True:
          top2.focus_force()
          return
       data_structures_displayed = True

       top2 = Toplevel()
       top2.title('Main Pack Grid Data Structures')
       top2.focus_force()
       display_info(top2, 'Data_Structures')
    return


# ======================================================================
# Display a formatted information ("help") window of Pack Grid
# documentation.

def display_info(topx, info):

   # Position the Toplevel window.
   textbox_width = 560
   str_width = str(textbox_width)
   str_height = str(topx.winfo_screenheight() -65) # Task bar adjustment
   if   topx == top1: shift = -200
   elif topx == top2: shift = 200
   else: raise 'Bad top window'
   x_offset = str((topx.winfo_screenwidth()  - textbox_width) // 2 + shift)
   y_offset = '0'
   geom = str_width + 'x' + str_height + '+' + x_offset + '+' + y_offset
   topx.geometry(geom)

   # Set up a buttons frame for the window.
   buttons = Frame(topx)
   buttons.pack(side=TOP, expand=NO, fill=X)

   # Set up the "exit program" (close all) button.
   exit_all = Button(buttons, text='CLOSE ALL "PACK GRID" WINDOWS',
                     font=('lucida', 9, 'bold'),
                     command=sys.exit, bg='red')
   exit_all.pack(side=LEFT, expand=YES)

   # Set up the "close just this window" button and the window closure
   # handler for it.
   def topx_destroy():
      global data_structures_displayed, general_info_displayed
      topx.destroy()
      if topx == top1: general_info_displayed = False
      if topx == top2: data_structures_displayed = False
   topx.protocol('WM_DELETE_WINDOW',topx_destroy)
   exit_info = Button(buttons, text='CLOSE JUST THIS WINDOW',
                      command=topx_destroy, bg='cyan')
   exit_info.pack(side=RIGHT, expand=YES)

   # Set up the left double-click window closure handler.
   def onDoubleLeftClick(event):
      global data_structures_displayed, general_info_displayed
      topx.destroy()
      if topx == top1: general_info_displayed = False
      if topx == top2: data_structures_displayed = False
   topx.bind('<Double-1>', onDoubleLeftClick)

   # Build the scrollable Text widget.
   textfrm = Frame(topx)
   textfrm.pack(side=TOP, expand=YES, fill=Y)
   sbar = Scrollbar(textfrm)
   txtw = Text(textfrm, padx=10)
   sbar.config(command=txtw.yview)
   txtw.config(yscrollcommand=sbar.set, bd=5, relief=FLAT)
   sbar.pack(side=RIGHT, fill=Y)
   txtw.pack(side=LEFT, expand=YES, fill=BOTH)

# ======================================================================
# These are general documentation formatting functions.

   # Build a title for the text display.

   def title(in_str):
      data_title = '\n' + in_str.strip() + '\n\n'
      txtw.insert(END, data_title, 'title')
      txtw.tag_config('title', font=('lucida', 12, 'bold'), foreground="blue",
                      justify=CENTER, underline=True)
      return

   # Build a proportional font, wrapped text section.  To allow wrapping
   # (text flow), create one long string with almost every '\n' replaced
   # by a space.  Two special cases when processing the split-data list:
   # 1) Just ignore the initial '' string that arises because of the
   # '\n' between the ''' and the first word on the next line at the
   # start of the docstring. 2) To allow embedded blank lines, replace
   # any embedded '' string with a '\n\n'.  Add a '\n\n' as the final
   # characters to separate this section from the following section.

   def propo(in_str):
      para = ''
      for line in in_str.split('\n'):
         if line == '':
            if para == '': pass
            else: para = para + '\n\n'
         else: para = para + line + ' '
      txtw.insert(END, para, 'top')
      txtw.tag_config('top', font=('lucida', 10,), wrap=WORD)
      return

   # Build an unwrapped, monofont section.  Two special cases for
   # handling a '\n': 1) Delete the first '\n', the one between the '''
   # and the first word on the next line of the docstring. 2) Leave any
   # embedded '\n' in place.  Add a '\n\n' as the final characters to
   # separate this section from the following section.

   def mono(in_str):
      data_asis = in_str.lstrip('\n').rstrip() + '\n\n'
      txtw.insert(END, data_asis, 'asis')
      txtw.tag_config('asis', font=('courier', 9,), foreground="blue")
      return
# ======================================================================


   # Get raw data from the appropriate documentation file and format it.
   if info == 'Data_Structures':
      title(pack_grid_documentation.title_data_top)
      propo(pack_grid_documentation.propo_data_top)
      mono( pack_grid_documentation.mono_data_asis)
      propo(pack_grid_documentation.propo_data_bottom)

   else:    # 'General Information'
      title(pack_grid_documentation.title_gen_top)
      propo(pack_grid_documentation.propo_gen_top)
      mono( pack_grid_documentation.mono_gen_asis)
      propo(pack_grid_documentation.propo_gen_bottom)

   # Put focus on the info display
   txtw.focus()

   return
